#ifndef DeviceDriver_h
#define DeviceDriver_h

#include <arduino.h>
#include <avr/pgmspace.h>
#include <stdlib.h>
#include <limits.h>

#include <Framework/ByteOrder.h>
#include <Framework/SemVer.h>
#include "ClientReporter.h"
#include "LogicalUnitInfo.h"
#include "DeviceError.h"

// Extract 7-bit index values from a 14-bit handle

#define getUnitNumber(handle) ((handle) & 0x7F)
#define getDeviceNumber(handle) (((handle) >> 7) & 0x7F)

// Open() flags

#define DDO_FORCE_OPEN 0x01

// These are the common register codes used by the DeviceDrivers in
// their status() (CSR) and control() (CCR) methods.  Register names specific
// to a particular device type are defined by the individual DeviceDrivers.
//
// NOTE: Since a device driver knows whether it is performing a status()
// call or a control() call independent of the register number, it is okay
// if the CSR and CCR numeric values and/or names overlap as they will
// not cause a conflict.
//
// NOTE:  The range of values -1..-255 is reserved for the common virtual
// register values.  Any virtual registers defined by a DeviceDriver must
// fall in one of the ranges 0..32767 or -256..-32768 to avoid conflict
// with the common registers.  Obviously, any positive numbered virtual
// registers must also avoid conflict with the actual hardware device
// register numbers.

enum class CSR : int {
    DriverVersion   = -1,   /* Get driver name and version */
    LibraryVersion  = -2,   /* Get library name and version */
    Configure       = -3,   /* Get configuration of a logical unit number instance */
    Intervals       = -4,   /* Get current timer intervals for this device */
    Debug           = -255  /* Do something helpful for debugging ... */
};

enum class CCR : int {
    Reset           = -1,   /* Reset all state in the device driver */
    Configure       = -3,   /* Set configuration of a logical unit number instance */
    Intervals       = -4,   /* Set current timer intervals for this device */
    Debug           = -255  /* Do something helpful for debugging ... */
};

#define MAX_ROOT_NAME_LENGTH 32

#define MINIMUM_UPDATE_INTERVAL 100    // microseconds
#define DEFAULT_UPDATE_INTERVAL 200

#define MINIMUM_REPORT_INTERVAL 10     // milliseconds
#define DEFAULT_REPORT_INTERVAL 19


class DeviceDriver {

    friend class DeviceTable;

public:

    DeviceDriver(const char *nameRoot, const int count);

    virtual int open(const char *name, int flags) = 0;
    virtual int status(int handle, int reg, int count, byte *buf) = 0;
    virtual int control(int handle, int reg, int count, byte *buf) = 0;
    virtual int read(int handle, int count, byte *buf) = 0;
    virtual int write(int handle, int count, byte *buf) = 0;
    virtual int close(int handle) = 0;

    virtual int processTimerEvent(int lun, int timerSelector, ClientReporter *r);

    int checkForTimerEvents(ClientReporter *r);

protected:

    int getFullHandle(int lun);

    const char *rootName;

    int deviceNumber;        // the major handle value, ie index in the DeviceTable
    int logicalUnitCount;
    LogicalUnitInfo **logicalUnits;

    int buildVersionResponse(const byte *semver, const char *name,
                             const char *prLabel, const char *bLabel, int count, byte *buf);

    int controlIntervals(int handle, int reg, int count, byte *buf);
    int statusIntervals(int handle, int reg, int count, byte *buf);

private:

    unsigned long calculateElapsedTime(LogicalUnitInfo *lui, int timerIndex);

};

#endif
